import requests
import base64
import os

RIVALZ_API_URL = "https://be.rivalz.ai"


class RivalzClient:
    def __init__(self, secret: str = ''):
        self.secret = secret or os.getenv('SECRET_TOKEN')

    def upload_file(self, file_path: str):
        with open(file_path, 'rb') as file:
            files = {'file': file}
            headers = {
                'Authorization': f'Bearer {self.secret}'
            }
            res = requests.post(f"{RIVALZ_API_URL}/api-v1/ipfs/upload-file", files=files, headers=headers)
            res.raise_for_status()  # Raise an error for bad status codes
            upload_hash = res.json()['data']['uploadHash']
            return self.get_ipfs_hash(upload_hash)

    def upload_passport(self, file_path: str):
        with open(file_path, 'rb') as file:
            files = {'file': file}
            headers = {
                'Authorization': f'Bearer {self.secret}'
            }
            res = requests.post(f"{RIVALZ_API_URL}/ipfs/upload-passport-image", files=files, headers=headers)
            res.raise_for_status()  # Raise an error for bad status codes
            upload_hash = res.json()['data']['uploadHash']
            return self.get_ipfs_hash(upload_hash)
   
    def download(self, ipfs_hash: str):
        headers = {
            'Authorization': f'Bearer {self.secret}'
        }
        res = requests.get(f"{RIVALZ_API_URL}/ipfs/download-file/{ipfs_hash}", headers=headers)
        res.raise_for_status()  # Raise an error for bad status codes        
        data = res.json()['data']
        file_data = data['file']['data']
        file_name = data['name']
        return file_data, file_name

    def delete_file(self, ipfs_hash: str):
        headers = {
            'Authorization': f'Bearer {self.secret}'
        }
        res = requests.post(f"{RIVALZ_API_URL}/ipfs/delete-file/{ipfs_hash}", headers=headers)
        res.raise_for_status()  # Raise an error for bad status codes
        return res.json()

    def get_ipfs_hash(self, upload_hash: str):
        headers = {
            'Authorization': f'Bearer {self.secret}'
        }
        while True:
            res = requests.get(f"{RIVALZ_API_URL}/upload-history/{upload_hash}", headers=headers)
            res.raise_for_status()  # Raise an error for bad status codes
            upload_info = res.json()['data']
            if upload_info:
                return upload_info


import unittest
from unittest.mock import patch, mock_open
from rivalz_client.client import RivalzClient
import base64
from dotenv import load_dotenv
import os
class TestRivalzClient(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        load_dotenv()
        cls.secret_token = os.getenv('SECRET_TOKEN')

    def setUp(self):
        self.client = RivalzClient(self.secret_token)
        self.sample_file_path = 'sample.txt'
        self.sample_ipfs_hash = 'QmSampleHash'
        self.sample_save_path = 'downloads'
        self.sample_file_content = 'This is a sample file content.'
        self.sample_response_data = {
            'data': {
                'file': base64.b64encode(self.sample_file_content.encode()).decode(),
                'name': 'sample.txt'
            }
        }

    @patch('builtins.open', new_callable=mock_open, read_data='file_content')
    @patch('requests.post')
    def test_upload_file(self, mock_post, mock_file):
        mock_post.return_value.json.return_value = {'status': 'success'}

        response = self.client.upload_file(self.sample_file_path)

        mock_file.assert_called_with(self.sample_file_path, 'rb')
        mock_post.assert_called_once()
        self.assertEqual(response, {'status': 'success'})

    @patch('builtins.open', new_callable=mock_open, read_data='file_content')
    @patch('requests.post')
    def test_upload_passport(self, mock_post, mock_file):
        mock_post.return_value.json.return_value = {'status': 'success'}

        response = self.client.upload_passport(self.sample_file_path)

        mock_file.assert_called_with(self.sample_file_path, 'rb')
        mock_post.assert_called_once()
        self.assertEqual(response, {'status': 'success'})

    @patch('requests.get')
    @patch('builtins.open', new_callable=mock_open


# setup.py

from setuptools import setup, find_packages

setup(
    name='rivalz-py-client',
    version='0.1.7',
    author='Bui Dinh Ngoc',
    author_email='buidinhngoc.aiti@gmail.com',
    description='A Python client for interacting with Rivalz API',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Rivalz-ai/pyhon-client/',
    packages=find_packages(),
    install_requires=[
        'requests>=2.28.2',
        'python-dotenv>=1.0.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)